package photons;

// import java.util.Random;

import espace3D.Vecteur3D;
import fonctions.Fonction1D;
import fonctions.Lorentz;
import fonctions.Gauss;
import fonctions.Tirages;
import fonctions.Constantes;

/**
 * classe rassemblant les donnes du dispositif laser
 * @author jeanpierretreuil
 * cette classe calcul en fonction des donnes de l'atome, 
 * les donnes du photon interagissant avec lui
 * (en terme de longueur d'onde, de direction et de dlai)
 * dans desexcitation stimule et dans l'excitation)
 *
 */
public class Dispositif_Laser {
    //---------
    //frquence centrale de chaque faisceau
    private double frequence;public double getfrequence() {return frequence;}//(en GHZ)
    //intensit "bolomtrique" de chaque faisceau, en erg/cm2 Ster-1
    private double intensite;public double getintensite() {return intensite;}
    //largeur de la raie de chaque faisceau en GHZ.
    private double largeur;public double getlargeur(){return largeur;}
  //nombre de faisceaux
    private int nbdir;
    //tableau des directions de chaque faisceau
    private Vecteur3D [] directions;public Vecteur3D[] getdirections(){return directions;}
    
    //---------------------------------------------------
    //nombre de frequences chantillonnes
    private int nbfreq;
    //tableau des frquences chantillonnes
    private double [] frequences;
    // tableau des intensits sur les frquences chantillonnes
    private double [] intensites;
    //tableau des prototypes des photons mis sur chaque faisceau 
    //et chaque frquence chantillonne
    private Photon[][] photons;
	//-------------------------
    public Dispositif_Laser(double lambdalaser,double intensitelaser, double largeurlaser, double sensibilite){
        this.frequence = Constantes.c/lambdalaser;
        this.intensite = intensitelaser;
        this.largeur = largeurlaser;
        //-----------
        this.nbdir = 6;//nombre de faisceaux laser utiliss (6 max)
        //----
		//initialisation du vecteur directeur de chacun des faisceaux
        directions = new Vecteur3D[6];
        directions[0] = new Vecteur3D(-1,0,0);
        directions[1] = new Vecteur3D( 1,0,0);
        directions[2] = new Vecteur3D(0,-1,0);
        directions[3] = new Vecteur3D(0, 1,0);
        directions[4] = new Vecteur3D(0,0,-1);
        directions[5] = new Vecteur3D(0,0, 1);
        //initialisation du profil discret de la raie laser
        // : tableaux des frquences chantillonnes et des intensits associes
        //Si la largeur  mi hauteur de la raie laser est infrieure au dixime
        //de la largeur  mi hauteur de la raie de rsonnance de l'atome
        //(ici enregistre sous l'attribut sensibilit)
        //alors on fait comme si la largeur de la raie laser tait nulle
        if(largeur < 0.052){
            nbfreq = 1;
            frequences = new double[1];
            intensites = new double[1];
            frequences[0]= frequence;
            intensites[0]= intensite;
        }else{
        	//calcul du pas d'chantillonnage
        	//et du nombre de frequences chantillonnes
        	double pas = 0.10*sensibilite;
            double fwhm = largeur*sensibilite;
            double coeff = 2*Math.sqrt(2*Math.log(2));
            double sigma = fwhm/coeff;
        	nbfreq = 2*((int) Math.ceil(3*sigma/pas))+1;
        	//construction du profil
        	Fonction1D gaussienne = new Gauss(frequence, sigma);
        	frequences = new double[nbfreq];
            intensites = new double[nbfreq];
            frequences[0]= frequence;
            intensites[0]= intensite*gaussienne.integrale(frequence-pas/2,frequence+pas/2, pas/4);
            int j = 1;
            for (int i = 1; i <= (nbfreq-1)/2  ; i++){
            	double freqp = frequence +i*pas;
            	double freqm = frequence -i*pas;
            	frequences[j]= freqp;
            	frequences[j+1]= freqm;
            	intensites[j]= intensite*gaussienne.integrale(freqp-pas/2, freqp+pas/2, pas/4);
            	intensites[j+1]= intensite*gaussienne.integrale(freqm-pas/2, freqm+pas/2, pas/4);
            	j=j+2;
            }
        }
        //initialisation du tableau des prototypes de photons 
        //mis sur pour chaque faisceau et chaque frequence echantillonnee
        photons = new Photon[nbfreq][nbdir];
        for (int i = 0; i<nbfreq ; i++){
        	for(int j = 0 ; j<nbdir ; j++){
        		photons[i][j]= new Photon(frequences[i], directions[j],0);
        	}
        }
        
        
    }
    /**
     * calcul des donnes du prochain photon
     * interagissant avec l'atome
     * @param freqres frquence de rsonnance de l'atome
     * @param vitesse vitesse 3D de l'atome
     * @param tau temps de dsexcitation de l'atome
     * On considre pour le moment que les paramtres sont les mmes
     * pour l'absorption et l'mission induite
     * On introduira s'il le faut les poids statistiques des niveaux
     * @return le photon inducteur ou excitateur
     */
    public Photon getphoton(Lorentz psi, Vecteur3D vitesse, double tauxRenc) {
        //----------------------------
        //calcul du dlai effectif 
        //de la premire interaction potentielle photon-atome 
        //pour chaque frquence chantillonne
        //et chaque faisceau
        //et enregistrement de ces dlais
    	//dans la foule, slection du photon "gagnant" ( dlai d'interaction le plus court)
    	double delairetenu = Double.MAX_VALUE;
        Photon photonretenu = null;
        for(int i = 0 ; i<nbfreq ; i++){
    		for( int j = 0 ; j<nbdir ;j++){
    		   double vitesseff = -vitesse.scalWith(directions[j]);
                   double delaimoyen = getdelaimoyen(psi, vitesseff, tauxRenc,   frequences[i], intensites[i]);
                   double delaieff = Tirages.tirage_exponentiel(delaimoyen);
                   photons[i][j].setdelai(delaieff);
                   if(delaieff<delairetenu){
                       delairetenu = delaieff;
                       photonretenu = photons[i][j];
                   }
                }
        }
        return photonretenu;
    }
    /**
     * calcul de l'intensit du processus de Poisson de la rpartition temporelle
     * des photons qui interagissent, puis du dlai d'attente moyen rsultant,
     * pour une frquence de laser et une intensit sur cette frquence donnes
     * le dlai d'attente est retourn en nanosecondes
     * @param freqres frquence de rsonnance de l'atome
     * @param vitesseff vitesse effective de l'atome (dans la direction d'un faisceau)
     * @param tau temps de dexcitation de l'atome
     * @param freqeff frquence du laser
     * @param intenseff intensit du laser sur cette frquence
     */
    private double getdelaimoyen(Lorentz psi, double vitesseff,double tauxRenc, double freqeff, double intenseff) {
        //prise en compte de l'effet Doppler
        double freqperue = freqeff*(1+(vitesseff/Constantes.c));
        //calcul de la probabilit d'interaction avec l'atome
        double proba = psi.f(freqperue);
        // dtermination de l'intensit du processus de Poisson
        //des photons ayant interagit avec l'atome
        //puis retour du dlai d'attente moyen
        double tauxeff = proba*tauxRenc*intenseff;
        return((1.0/tauxeff )*1.0E9);
    }
}
